unit Main;

{
  This demo shows how to use DSPLAB inside VST-plugins.
  The DSPLAB_Timbre project is an implementation of timbre block with
  lowpass (bass) and highpass (treble) filters. The plugin allows to change
  the cutoff frequency of the lowpass (50-500 Hz) and highpass (3-10 kHz)
  filters. The range of amplification from -20 db upto +20 db for each filter.

  ADDITIONAL RESOURCES

  The demo uses a Delphi VST SDK library. It is located at DVSTSDK folder.
  Thanks to Frederic Vanmol (frederic@axiworld.be) for this work.
  Please visit to http://www.axiworld.be to get latest version of the library.


  HOW TO INSTALL

  After building you will have a DSPLAB_Timbre.dll.
  This is a VST-plugin. Please refer to guide of your software
  to install the VST-plugin properly.

  CONTACT INFORMATION

  Author: Eugen Tarasov
  e-mail: dsplab@teworks.com
  http://www.teworks.com
}


interface

uses DAEffect, DAudioE, VSTUtils, dspIIRFilters, dspTypes, Math, SysUtils;

const
  cNumParams = 4;
  cDefSampleRate = 44100;
  cDefLowpassFreq = 50;
  cDefHighpassFreq = 10000;

type
  TPluginsParams = (cLowFreq, cHighFreq, cLowFreqGain, cHighFreqGain);

  TDsplabEffect = class(AudioEffect)
  private
    FLowpassGain, FHighpassGain: Single;
    LowpassFilter1, HighpassFilter1: TdspIIRFilter;
    LowpassFilter2, HighpassFilter2: TdspIIRFilter;

    procedure FSetLowFreq(V: Single; Translate: Boolean = true);
    procedure FSetHighFreq(V: Single; Translate: Boolean = true);

  public
    constructor Create(audioMaster: TAudioMasterCallbackFunc; numPrograms,
                       numParams: Longint); override;
    destructor Destroy; override;

    procedure setSampleRate(sampleRate: Single); override;

    procedure process(inputs, outputs: PPSingle; sampleframes: Longint); override;
    procedure processReplacing(inputs, outputs: PPSingle; sampleframes: Longint); override;

    procedure setParameter(index: Longint; value: Single); override;
    function  getParameter(index: Longint): Single; override;
    procedure getParameterLabel(index: Longint; aLabel: PChar); override;
    procedure getParameterDisplay(index: Longint; text: PChar); override;
    procedure getParameterName(index: Longint; text: PChar); override;
  end;


implementation


constructor TDsplabEffect.Create(audioMaster: TAudioMasterCallbackFunc;
                                  numPrograms, numParams: Longint);
begin
  inherited Create(audioMaster, 1, cNumParams);

  { Setup a timbre block }
  FLowpassGain:= 0;
  FHighpassGain:= 0;

  LowpassFilter1:= TdspIIRFilter.Create(nil);
  LowpassFilter1.Kind:= fkButterworth;
  LowpassFilter1.Response:= ftLowpass;
  LowpassFilter1.Order:= 1;

  HighpassFilter1:= TdspIIRFilter.Create(nil);
  HighpassFilter1.Kind:= fkButterworth;
  HighpassFilter1.Response:= ftHighpass;
  HighpassFilter1.Order:= 1;

  LowpassFilter2:= TdspIIRFilter.Create(nil);
  LowpassFilter2.Kind:= fkButterworth;
  LowpassFilter2.Response:= ftLowpass;
  LowpassFilter2.Order:= 1;

  HighpassFilter2:= TdspIIRFilter.Create(nil);
  HighpassFilter2.Kind:= fkButterworth;
  HighpassFilter2.Response:= ftHighpass;
  HighpassFilter2.Order:= 1;

  setSampleRate(cDefSampleRate);
  FSetLowFreq(cDefLowpassFreq, false);
  FSetHighFreq(cDefHighpassFreq, false);

  {Setup a VST }
  setProgram(0);
  setNumInputs(2);
  setNumOutputs(2);
  canMono(true);
  canProcessReplacing(TRUE);
  setUniqueID(FourCharToLong('D', 'S', 'P', 'T'));
end;


destructor TDsplabEffect.Destroy;
begin
  LowpassFilter1.Free;
  HighpassFilter1.Free;
  LowpassFilter2.Free;
  HighpassFilter2.Free;

  inherited Destroy;
end;


procedure TDsplabEffect.setSampleRate(sampleRate: Single);
begin
  LowpassFilter1.SampleRate  := sampleRate;
  HighpassFilter1.SampleRate := sampleRate;
  LowpassFilter2.SampleRate  := sampleRate;
  HighpassFilter2.SampleRate := sampleRate;
end;


procedure TDsplabEffect.FSetLowFreq(V: Single; Translate: Boolean = true);
begin
  // Translate 0-1 to 50-500
  if Translate then V:= 50 + 450 * V;
  LowpassFilter1.Frequency1:= V;
  LowpassFilter2.Frequency1:= V;
end;


procedure TDsplabEffect.FSetHighFreq(V: Single; Translate: Boolean = true);
begin
  // Translate 0-1 to 3000-10000
  if Translate then V:= 3000 + 7000 * V;
  HighpassFilter1.Frequency1:= V;
  HighpassFilter2.Frequency1:= V;
end;


procedure TDsplabEffect.setParameter(index: Longint; value: Single);
  function TranslateGain(V: Single): Single;
  begin
    // Translate 0-1 to -0.9..+9 (-20..+20 dB)
    Result:= Power(10, 2 * V - 1) - 1;
  end;
begin
  case TPluginsParams(index) of
    cLowFreq:  FSetLowFreq(value);
    cHighFreq:  FSetHighFreq(value);
    cLowFreqGain: FLowpassGain:= TranslateGain(value);
    cHighFreqGain: FHighpassGain:= TranslateGain(value);
  end;
end;


function TDsplabEffect.getParameter(index: Longint): Single;
  function TranslateGain(V: Single): Single;
  begin
    Result:= (log10(V + 1) + 1) / 2;
  end;
begin
  case TPluginsParams(index) of
    cLowFreq:  Result:= (LowpassFilter1.Frequency1 - 50) / 450;
    cHighFreq: Result:= (HighpassFilter1.Frequency1 - 3000) / 7000;
    cLowFreqGain:  Result:= TranslateGain(FLowpassGain);
    cHighFreqGain: Result:= TranslateGain(FHighpassGain);
  else
    Result:= 0;
  end;
end;


procedure TDsplabEffect.getParameterName(index: Longint; text: PChar);
begin
  case TPluginsParams(index) of
    cLowFreq:  StrCopy(text, 'Bass cutoff');
    cHighFreq: StrCopy(text, 'Treble cutoff');
    cLowFreqGain:  StrCopy(text, 'Bass');
    cHighFreqGain: StrCopy(text, 'Treble');
  end;
end;


procedure TDsplabEffect.getParameterDisplay(index: Longint; text:PChar);
begin
  case TPluginsParams(index) of
    cLowFreq:  float2string(LowpassFilter1.Frequency1, text);
    cHighFreq: float2string(HighpassFilter1.Frequency1, text);
    cLowFreqGain:  dB2string(FLowpassGain + 1, text);
    cHighFreqGain: dB2string(FHighpassGain + 1, text);
  end;
end;


procedure TDsplabEffect.getParameterLabel(index:Longint; aLabel: PChar);
begin
  case TPluginsParams(index) of
    cLowFreq,
    cHighFreq: StrCopy(aLabel, '  Hz    ');
    cLowFreqGain,
    cHighFreqGain: StrCopy(aLabel, '  dB    ');
  end;
end;

//--------------------------------------------------------------------------------

procedure TDsplabEffect.process(inputs, outputs: PPSingle; sampleframes: Longint);
var
  I: Integer;
  In1, In2, Out1, Out2: PSingle;
begin
  In1:= Inputs^;
  Inc(Inputs);
  In2:= Inputs^;

  Out1:= Outputs^;
  Inc(Outputs);
  Out2:= Outputs^;

  for I:= 1 to SampleFrames do
  begin
    Out1^:= Out1^ + In1^ +
            FLowpassGain * LowpassFilter1.Filter(In1^) +
            FHighpassGain * HighpassFilter1.Filter(In1^);

    Out2^:= Out2^ + In2^ +
            FLowpassGain * LowpassFilter2.Filter(In2^) +
            FHighpassGain * HighpassFilter2.Filter(In2^);

    Inc(In1); Inc(Out1);
    Inc(In2); Inc(Out2);
  end;
end;


procedure TDsplabEffect.processReplacing(inputs, outputs: PPSingle; sampleframes: Longint);
var
  I: Integer;
  In1, In2, Out1, Out2: PSingle;
begin
  In1:= Inputs^;
  Inc(Inputs);
  In2:= Inputs^;

  Out1:= Outputs^;
  Inc(Outputs);
  Out2:= Outputs^;

  for I:= 1 to SampleFrames do
  begin
    Out1^:= In1^ +
            FLowpassGain * LowpassFilter1.Filter(In1^) +
            FHighpassGain * HighpassFilter1.Filter(In1^);

    Out2^:= In2^ +
            FLowpassGain * LowpassFilter2.Filter(In2^) +
            FHighpassGain * HighpassFilter2.Filter(In2^);

    Inc(In1); Inc(Out1);
    Inc(In2); Inc(Out2);
  end;
end;


end.
